@@ -29,11 +29,13 @@
 #include <Lum/Button.h>
 #include <Lum/Dialog.h>
 #include <Lum/Panel.h>
-#include <Lum/PopupGroup.h>
+#include <Lum/Popup.h>
 #include <Lum/Text.h>
 #include <Lum/Toggle.h>
 
-class SetSelector : public Lum::Dialog
+static size_t popupFontSize = 200;
+
+class SetSelector : public Lum::Popup
 {
 private:
   Lum::Model::ActionRef models[10]; // Values 0..9, where 0 means empty
@@ -49,23 +51,13 @@ public:
       models[i]=new Lum::Model::Action();
       AttachModel(models[i]);
     }
-
-    SetType(Lum::OS::Window::typePopup);
-  }
-
-  ~SetSelector()
-  {
-    for (size_t i=0; i<=9; i++) {
-      UnattachModel(models[i]);
-    }
   }
 
   void PreInit()
   {
-    Lum::Array      *array;
-    Lum::Button     *button;
-    Lum::Panel      *panel;
-    Lum::PopupGroup *container;
+    Lum::Array  *array;
+    Lum::Button *button;
+    Lum::Panel  *panel;
 
     panel=new Lum::VPanel();
 
@@ -80,7 +72,7 @@ public:
       text->SetFlex(true,true);
       text->SetText(Lum::Base::NumberToWString(i),
                     Lum::OS::display->GetFont(Lum::OS::Display::fontTypeProportional,
-                                              Lum::OS::Display::fontScaleCaption1));
+                                              popupFontSize));
       text->SetAlignment(Lum::Text::centered);
 
       button=new Lum::Button();
@@ -99,12 +91,9 @@ public:
 
     panel->Add(button);
 
-    container=new Lum::PopupGroup();
-    container->SetMain(panel);
+    SetMain(panel);
 
-    SetTop(container);
-
-    Dialog::PreInit();
+    Popup::PreInit();
   };
 
   void Resync(Lum::Base::Model *model, const Lum::Base::ResyncMsg& msg)
@@ -118,7 +107,7 @@ public:
       }
     }
 
-    Dialog::Resync(model,msg);
+    Popup::Resync(model,msg);
   }
 
   bool HasSelection() const
@@ -132,7 +121,7 @@ public:
   }
 };
 
-class MarkSelector : public Lum::Dialog
+class MarkSelector : public Lum::Popup
 {
 private:
   Lum::Model::BooleanRef models[9];
@@ -146,22 +135,12 @@ public:
       models[i]=new Lum::Model::Boolean(value & (1 << i));
       AttachModel(models[i]);
     }
-
-    SetType(Lum::OS::Window::typePopup);
-  }
-
-  ~MarkSelector()
-  {
-    for (size_t i=0; i<=8; i++) {
-      UnattachModel(models[i]);
-    }
   }
 
   void PreInit()
   {
-    Lum::Array      *array;
-    Lum::Panel      *panel;
-    Lum::PopupGroup *container;
+    Lum::Array *array;
+    Lum::Panel *panel;
 
     panel=new Lum::VPanel();
 
@@ -177,7 +156,7 @@ public:
       text->SetFlex(true,true);
       text->SetText(Lum::Base::NumberToWString(i+1),
                     Lum::OS::display->GetFont(Lum::OS::Display::fontTypeProportional,
-                                              Lum::OS::Display::fontScaleCaption1));
+                                              popupFontSize));
       text->SetAlignment(Lum::Text::centered);
 
       toggle=new Lum::Toggle();
@@ -189,12 +168,9 @@ public:
     }
     panel->Add(array);
 
-    container=new Lum::PopupGroup();
-    container->SetMain(panel);
+    SetMain(panel);
 
-    SetTop(container);
-
-    Dialog::PreInit();
+    Popup::PreInit();
   };
 
   void Resync(Lum::Base::Model *model, const Lum::Base::ResyncMsg& msg)
@@ -212,7 +188,7 @@ public:
       }
     }
 
-    Dialog::Resync(model,msg);
+    Popup::Resync(model,msg);
   }
 
   size_t GetValue() const
@@ -428,8 +404,10 @@ void GameArea::Draw(int x, int y, size_t w, size_t h)
     DrawBackground(this->x+size,this->y+size,width-size,height-size);
   }
 
+  //
+  // Draw game area
+  //
   draw->PushForeground(Lum::OS::display->blackColor);
-
   offset=0;
   for (size_t i=0; i<10; i++) {
     // Vertical
@@ -457,23 +435,28 @@ void GameArea::Draw(int x, int y, size_t w, size_t h)
 
   font=FindFont(boxSize-2*Lum::OS::display->GetSpaceHorizontal(Lum::OS::Display::spaceObjectBorder));
 
+  //
+  // Draw values
+  //
   draw->PushFont(font,Lum::OS::Font::bold);
   for (size_t i=0; i<Sudoku::dimension*Sudoku::dimension; i++) {
+
+    // Draw background
+    if (HasFocus() && !finished->Get() && i==focusedCell) {
+      draw->PushForeground(focusBackgroundColor);
+    }
+    else {
+      draw->PushForeground(Lum::OS::display->whiteColor);
+    }
+    draw->FillRectangle(this->x+GetCellOffset(i%Sudoku::dimension),
+                        this->y+GetCellOffset(i/Sudoku::dimension),
+                        boxSize,boxSize);
+    draw->PopForeground();
+
     if (riddle.Get(i)!=Sudoku::empty) { // Preset fields
       std::wstring        text(Lum::Base::NumberToWString(riddle.Get(i)));
       Lum::OS::FontExtent extent;
 
-      if (HasFocus() && !finished->Get() && i==focusedCell) { // Draw focused
-        draw->PushForeground(focusBackgroundColor);
-      }
-      else {
-        draw->PushForeground(Lum::OS::display->whiteColor);
-      }
-      draw->FillRectangle(this->x+GetCellOffset(i%Sudoku::dimension),
-                          this->y+GetCellOffset(i/Sudoku::dimension),
-                          boxSize,boxSize);
-      draw->PopForeground();
-
       font->StringExtent(text,extent,Lum::OS::Font::bold);
 
       draw->PushForeground(presetColor);
@@ -482,93 +465,74 @@ void GameArea::Draw(int x, int y, size_t w, size_t h)
                        text);
       draw->PopForeground();
     }
-    else {
-      // Draw background
-      if (HasFocus() && !finished->Get() && i==focusedCell) {
-        draw->PushForeground(focusBackgroundColor);
+    else if (finished->Get() && solution.Get(i)!=Sudoku::empty) {
+      std::wstring        text(Lum::Base::NumberToWString(solution.Get(i)));
+      Lum::OS::FontExtent extent;
+
+      font->StringExtent(text,extent,Lum::OS::Font::bold);
+
+      if (solution.Get(i)!=game.Get(i)) {
+        draw->PushForeground(errorColor);
       }
       else {
-        draw->PushForeground(Lum::OS::display->whiteColor);
+        draw->PushForeground(correctColor);
       }
-      draw->FillRectangle(this->x+GetCellOffset(i%Sudoku::dimension),
-                          this->y+GetCellOffset(i/Sudoku::dimension),
-                          boxSize,boxSize);
+
+      draw->DrawString(this->x+GetCellOffset(i%Sudoku::dimension)+(boxSize-extent.width+extent.left+extent.right)/2-extent.left,
+                       this->y+GetCellOffset(i/Sudoku::dimension)+(boxSize-extent.height)/2+extent.ascent,
+                       text);
       draw->PopForeground();
+    }
+    else if (game.Get(i)!=Sudoku::empty) { // manually set fields
+      std::wstring         text(Lum::Base::NumberToWString(game.Get(i)));
+      Lum::OS::FontExtent  extent;
+      Sudoku::Bitset       allowed;
 
-      if (game.Get(i)!=Sudoku::empty || finished->Get()) { // manually set fields
-        std::wstring         text;
-        Lum::OS::FontExtent  extent;
-        Sudoku::Bitset       allowed;
+      game.GetPotential(i,allowed);
 
+      font->StringExtent(text,extent,Lum::OS::Font::bold);
 
-        if (finished->Get()) {
-          text=Lum::Base::NumberToWString(solution.Get(i));
-
-          if (solution.Get(i)!=game.Get(i)) {
-            draw->PushForeground(errorColor);
-          }
-          else {
-            draw->PushForeground(correctColor);
-          }
-        }
-        else {
-          text=Lum::Base::NumberToWString(game.Get(i));
-
-          game.GetPotential(i,allowed);
-
-          font->StringExtent(text,extent);
-
-          if (!allowed.test(game.Get(i))) {
-            draw->PushForeground(errorColor);
-          }
-          else {
-            draw->PushForeground(Lum::OS::display->blackColor);
-          }
-        }
-
-        draw->DrawString(this->x+GetCellOffset(i%Sudoku::dimension)+(boxSize-extent.width+extent.left+extent.right)/2-extent.left,
-                         this->y+GetCellOffset(i/Sudoku::dimension)+(boxSize-extent.height)/2+extent.ascent,
-                         text);
-        draw->PopForeground();
+      if (!allowed.test(game.Get(i))) {
+        draw->PushForeground(errorColor);
       }
       else {
-        for (size_t j=0; j<=8; j++) {
-          if (marks[i] & (1 << j)) {
-            std::wstring         text(Lum::Base::NumberToWString(j+1));
-            Lum::OS::FontExtent  extent;
+        draw->PushForeground(Lum::OS::display->blackColor);
+      }
 
-            markFont->StringExtent(text,extent);
+      draw->DrawString(this->x+GetCellOffset(i%Sudoku::dimension)+(boxSize-extent.width+extent.left+extent.right)/2-extent.left,
+                       this->y+GetCellOffset(i/Sudoku::dimension)+(boxSize-extent.height)/2+extent.ascent,
+                       text);
+      draw->PopForeground();
+    }
+    else {
+      for (size_t j=0; j<=8; j++) {
+        if (marks[i] & (1 << j)) {
+          std::wstring         text(Lum::Base::NumberToWString(j+1));
+          Lum::OS::FontExtent  extent;
 
-            draw->PushFont(markFont);
-            draw->PushForeground(Lum::OS::display->blackColor);
-            draw->DrawString(this->x+GetCellOffset(i%Sudoku::dimension)+
-                             (boxSize/3)*(j%3)+
-                             (boxSize/3-extent.width+extent.left+extent.right)/2-extent.left,
-                             this->y+GetCellOffset(i/Sudoku::dimension)+
-                             (boxSize/3)*(j/3)+
-                             (boxSize/3-extent.height)/2+extent.ascent,
-                             text);
-            draw->PopForeground();
-            draw->PopFont();
-          }
+          markFont->StringExtent(text,extent);
+
+          draw->PushFont(markFont);
+          draw->PushForeground(Lum::OS::display->blackColor);
+          draw->DrawString(this->x+GetCellOffset(i%Sudoku::dimension)+
+                           (boxSize/3)*(j%3)+
+                           (boxSize/3-extent.width+extent.left+extent.right)/2-extent.left,
+                           this->y+GetCellOffset(i/Sudoku::dimension)+
+                           (boxSize/3)*(j/3)+
+                           (boxSize/3-extent.height)/2+extent.ascent,
+                           text);
+          draw->PopForeground();
+          draw->PopFont();
         }
       }
     }
   }
   draw->PopFont();
+}
 
-  /*
-  if (HasFocus() && !finished->Get()) {
-    draw->PushForeground(Lum::OS::display->blackColor);
-    draw->PushDash("\002\002",2,::Lum::OS::DrawInfo::fDash);
-    draw->DrawRectangle(this->x+GetCellOffset(focusedCell%Sudoku::dimension)+Lum::OS::display->GetSpaceHorizontal(Lum::OS::Display::spaceIntraObject),
-                        this->y+GetCellOffset(focusedCell/Sudoku::dimension)+Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceIntraObject),
-                        boxSize-2*Lum::OS::display->GetSpaceHorizontal(Lum::OS::Display::spaceIntraObject),
-                        boxSize-2*Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceIntraObject));
-    draw->PopDash();
-    draw->PopForeground();
-  }*/
-
+bool GameArea::HandlesFocus() const
+{
+  return Object::HandlesFocus() && !finished->Get();
 }
 
 bool GameArea::HandleMouseEvent(const Lum::OS::MouseEvent& event)
