@@ -25,6 +25,7 @@
 #include <Lum/Base/String.h>
 
 #include <Lum/Dlg/About.h>
+#include <Lum/Dlg/Help.h>
 #include <Lum/Dlg/Msg.h>
 #include <Lum/Dlg/Progress.h>
 
@@ -37,7 +38,6 @@
 #include <Lum/Panel.h>
 #include <Lum/Space.h>
 #include <Lum/Text.h>
-#include <Lum/WindowGroup.h>
 
 #include <Lum/Model/Action.h>
 #include <Lum/Model/String.h>
@@ -140,33 +140,14 @@ public:
     AttachModel(gameArea->GetFinishedModel());
   }
 
-  ~MainDialog()
-  {
-    UnattachModel(gameArea->GetFinishedModel());
-    UnattachModel(generateTask.GetFinishedAction());
-    UnattachModel(GetClosedAction());
-    UnattachModel(GetOpenedAction());
-    UnattachModel(aboutAction);
-    UnattachModel(helpAction);
-    UnattachModel(statAction);
-    UnattachModel(solveAction);
-    UnattachModel(generateAction);
-  }
-
   void PreInit()
   {
     Lum::Array               *array;
     Lum::Button              *button;
     Lum::Combo               *combo;
     Lum::Label               *label;
-    Lum::Menu                *menu;
-    Lum::MenuStrip           *menuStrip;
     Lum::Model::ListTableRef table;
     Lum::Panel               *horiz,*vert;
-    Lum::WindowGroup         *wGroup;
-
-    wGroup=new Lum::WindowGroup();
-    wGroup->SetFlex(true,true);
 
     horiz=new Lum::HPanel();
     horiz->SetFlex(true,true);
@@ -273,30 +254,25 @@ public:
 
     horiz->Add(vert);
 
-    wGroup->SetMain(horiz);
+    SetMain(horiz);
 
-    menuStrip=new Lum::MenuStrip();
+    Lum::MenuDesc *menu=new Lum::MenuDesc();
 
-    menu=new Lum::Menu();
-    menu->AddActionItem(_ld(menuProjectQuit),GetClosedAction(),Lum::OS::qualifierControl,L"q");
-    menuStrip->AddPullDownMenu(_ld(menuProject),menu);
+    menu
+      ->AddMenuItemSub(_ld(menuProject))
+        ->AddMenuItemAction(_ld(menuProjectQuit),Lum::OS::qualifierControl,L"q",GetClosedAction());
+    menu
+      ->AddMenuItemSub(_(L"MENU_GAME",L"_Game"))
+        ->AddMenuItemAction(_(L"MENU_GAME_NEW",L"_Generate new game"),Lum::OS::qualifierControl,L"g",generateAction)
+        ->AddMenuItemAction(_(L"MENU_GAME_SOLVE",L"_Solve game"),Lum::OS::qualifierControl,L"s",solveAction)
+        ->AddSeparator()
+        ->AddMenuItemAction(_(L"MENU_GAME_STAT",L"S_tatistics"),Lum::OS::qualifierControl,L"t",statAction);
+    menu
+      ->AddMenuItemSub(_ld(menuHelp))
+        ->AddMenuItemAction(_ld(menuHelpHelp),helpAction)
+        ->AddMenuItemAction(_ld(menuHelpAbout),aboutAction);
 
-    menu=new Lum::Menu();
-    menu->AddActionItem(_(L"MENU_GAME_NEW",L"_Generate new game"),generateAction,Lum::OS::qualifierControl,L"g");
-    menu->AddActionItem(_(L"MENU_GAME_SOLVE",L"_Solve game"),solveAction,Lum::OS::qualifierControl,L"s");
-    menu->AddSeparator();
-    menu->AddActionItem(_(L"MENU_GAME_STAT",L"S_tatistics"),statAction,Lum::OS::qualifierControl,L"t");
-    menuStrip->AddPullDownMenu(_(L"MENU_GAME",L"_Game"),menu);
-
-    menu=new Lum::Menu();
-    menu->AddActionItem(_ld(menuHelpHelp),helpAction);
-    menu->AddSeparator();
-    menu->AddActionItem(_ld(menuHelpAbout),aboutAction);
-    menuStrip->AddPullDownMenu(_ld(menuHelp),menu);
-
-    wGroup->SetMenuStrip(menuStrip);
-
-    SetTop(wGroup);
+    SetMenu(menu);
 
     Dialog::PreInit();
   }
@@ -366,6 +342,9 @@ public:
           game=NULL;
         }
       }
+      if (gameArea->GetFinishedModel()->Get()) {
+        solveAction->Disable();
+      }
       gameArea->GetFinishedModel()->On(false);
     }
     else if (model==GetClosedAction() && GetClosedAction()->IsFinished()) {
@@ -379,13 +358,23 @@ public:
       GenerateRiddle();
     }
     if (model==generateTask.GetFinishedAction() && generateTask.GetFinishedAction()->IsFinished()) {
+      solveAction->Enable();
       SetRiddle();
       gameArea->GetFinishedModel()->On(false);
     }
-    else if (model==solveAction && solveAction->IsFinished()) {
-      gameArea->GetFinishedModel()->Off();
-      gameArea->Solve();
-      gameArea->GetFinishedModel()->On(false);
+    else if (model==solveAction && solveAction->IsFinished() && solveAction->IsEnabled()) {
+      if (Lum::Dlg::Msg::Ask(this,
+                             _(L"SOLVE_TITLE",L"Are you sure?"),
+                             _(L"SOLVE_BODY",
+                               L"Are you sure that you want to solve the game? \n"
+                               L"\n"
+                               L"Solving the game cannot be undone!"),
+                             _(L"SOLVE_BUTTONS",L"_Solve it!*|_Cancel^"))==0) {
+        gameArea->GetFinishedModel()->Off();
+        gameArea->Solve();
+        gameArea->GetFinishedModel()->On(false);
+        solveAction->Disable();
+      }
     }
     else if (model==statAction && statAction->IsFinished()) {
       Sudoku game;
@@ -395,18 +384,22 @@ public:
       StatisticsDialog::Show(GetWindow(),game);
     }
     else if (model==helpAction && helpAction->IsFinished()) {
-      Lum::Dlg::Msg::ShowOk(this,
-                            _(L"HELP_TITLE",L"How to use it?"),
-                            _(L"HELP_BODY",
-                              L"The interface supports setting numbers and marking fields as\n"
-                              L"potentially holding a subset of numbers.\n"
-                              L"\n"
-                              L"To mark a field click on the upperleft quadrant. A popup will open,\n"
-                              L"from which you can select/unselect multiple numbers for marking.\n"
-                              L"To set a number, click on any other quadrant. Again a similar dialog\n"
-                              L"will open, this time only allowing one (or no) number. Marks will\n"
-                              L"shown a small numbers, set values as black, predefined number will be\n"
-                              L"drawn grey, and conflicts red.  Have fun :-)"));
+      Lum::Dlg::Help::Show(this,
+                           _(L"HELP_BODY",
+                             L"The interface supports setting numbers and marking\n"
+                             L"fields as potentially holding a subset of numbers.\n"
+                             L"\n"
+                             L"To mark a field click on the upperleft quadrant. A\n"
+                             L"popup will open, from which you can select/unselect\n"
+                             L"multiple numbers for marking.\n"
+                             L"To set a number, click on any other quadrant. Again\n"
+                             L"a similar dialog will open, this time only allowing\n"
+                             L"one (or no) number.\n"
+                             L"Marks will shown a small numbers, set values as black,\n"
+                             L"predefined number will be drawn grey, and conflicts\n"
+                             L"red.\n"
+                             L"\n"
+                             L"Have fun :-)"));
     }
     else if (model==aboutAction && aboutAction->IsFinished()) {
       Lum::Dlg::About::Show(this,info);
@@ -418,6 +411,7 @@ public:
                               L"It looks like you have successfully finished\n"
                               L"the given riddle!\n"
                               L"Congratulations!"));
+      solveAction->Disable();
     }
 
 
